@@ -18,10 +18,15 @@
 #include <lfortran/pickle.h>
 #include <libasr/pickle.h>
 #include <libasr/utils.h>
+#include "libasr/asr_lookup_name.cpp"
+
 
 #ifdef HAVE_LFORTRAN_LLVM
 #include <libasr/codegen/evaluator.h>
 #include <libasr/codegen/asr_to_llvm.h>
+#ifdef HAVE_LFORTRAN_MLIR
+#include <libasr/codegen/asr_to_mlir.h>
+#endif
 #else
 namespace LCompilers {
     class LLVMEvaluator {};
@@ -125,33 +130,37 @@ Result<FortranEvaluator::EvalResult> FortranEvaluator::evaluate(
     // LLVM -> Machine code -> Execution
     e->add_module(std::move(m));
     if (return_type == "integer4") {
-        int32_t r = e->int32fn(run_fn);
+        int32_t r = e->execfn<int32_t>(run_fn);
         result.type = EvalResult::integer4;
         result.i32 = r;
     } else if (return_type == "integer8") {
-        int64_t r = e->int64fn(run_fn);
+        int64_t r = e->execfn<int64_t>(run_fn);
         result.type = EvalResult::integer8;
         result.i64 = r;
     } else if (return_type == "real4") {
-        float r = e->floatfn(run_fn);
+        float r = e->execfn<float>(run_fn);
         result.type = EvalResult::real4;
         result.f32 = r;
     } else if (return_type == "real8") {
-        double r = e->doublefn(run_fn);
+        double r = e->execfn<double>(run_fn);
         result.type = EvalResult::real8;
         result.f64 = r;
     } else if (return_type == "complex4") {
-        std::complex<float> r = e->complex4fn(run_fn);
+        std::complex<float> r = e->execfn<std::complex<float>>(run_fn);
         result.type = EvalResult::complex4;
         result.c32.re = r.real();
         result.c32.im = r.imag();
     } else if (return_type == "complex8") {
-        std::complex<double> r = e->complex8fn(run_fn);
+        std::complex<double> r = e->execfn<std::complex<double>>(run_fn);
         result.type = EvalResult::complex8;
         result.c64.re = r.real();
         result.c64.im = r.imag();
+    } else if (return_type == "logical") {
+        bool r = e->execfn<bool>(run_fn);
+        result.type = EvalResult::boolean;
+        result.b = r;
     } else if (return_type == "void") {
-        e->voidfn(run_fn);
+        e->execfn<void>(run_fn);
         result.type = EvalResult::statement;
     } else if (return_type == "none") {
         result.type = EvalResult::none;
@@ -233,6 +242,16 @@ Result<std::string> FortranEvaluator::get_asr(const std::string &code,
     }
 }
 
+LCompilers::ASR::asr_t* FortranEvaluator::handle_lookup_name(LCompilers::ASR::TranslationUnit_t* tu, uint64_t pos) {
+    LCompilers::LFortran::LookupNameVisitor lnv(pos);
+    lnv.visit_TranslationUnit(*tu);
+    if (lnv.node_to_return != nullptr) {
+        return lnv.node_to_return;
+    } else {
+        return ( LCompilers::ASR::asr_t*) tu;
+    }
+}
+
 Result<ASR::TranslationUnit_t*> FortranEvaluator::get_asr2(
             const std::string &code_orig, LocationManager &lm,
             diag::Diagnostics &diagnostics)
@@ -247,7 +266,6 @@ Result<ASR::TranslationUnit_t*> FortranEvaluator::get_asr2(
         LCOMPILERS_ASSERT(diagnostics.has_error())
         return res.error;
     }
-
     // AST -> ASR
     Result<ASR::TranslationUnit_t*> res2 = get_asr3(*ast, diagnostics);
     if (res2.ok) {
@@ -332,7 +350,7 @@ Result<std::unique_ptr<LLVMModule>> FortranEvaluator::get_llvm3(
     ASR::TranslationUnit_t &/*asr*/, LCompilers::PassManager &/*pass_manager*/,
     diag::Diagnostics &/*diagnostics*/
 #endif
-, const std::string &infile)
+, [[maybe_unused]] const std::string &infile)
 {
 #ifdef HAVE_LFORTRAN_LLVM
     eval_count++;
@@ -510,6 +528,54 @@ Result<std::string> FortranEvaluator::get_julia(const std::string &code,
     }
 }
 
+Result<std::string> FortranEvaluator::get_mlir(const std::string &code,
+    LocationManager &lm, diag::Diagnostics &diagnostics)
+{
+    // Src -> AST -> ASR -> MLIR
+    SymbolTable *old_symbol_table = symbol_table;
+    symbol_table = nullptr;
+    Result<ASR::TranslationUnit_t*> asr = get_asr2(code, lm, diagnostics);
+    symbol_table = old_symbol_table;
+    if (asr.ok) {
+#ifdef HAVE_LFORTRAN_MLIR
+        Result<std::unique_ptr<MLIRModule>> res = asr_to_mlir(al, *asr.result,
+            diagnostics);
+        return res.result->str();
+#else
+        throw LCompilersException("MLIR is not enabled");
+#endif
+    } else {
+        LCOMPILERS_ASSERT(diagnostics.has_error())
+        return asr.error;
+    }
+}
+
+Result<std::unique_ptr<MLIRModule>> FortranEvaluator::get_mlir2(
+#ifdef HAVE_LFORTRAN_MLIR
+        ASR::TranslationUnit_t &asr, diag::Diagnostics &diagnostics
+#else
+        ASR::TranslationUnit_t &/*asr*/, diag::Diagnostics &/*diagnostics*/
+#endif
+) {
+#ifdef HAVE_LFORTRAN_MLIR
+    // ASR -> MLIR
+    std::unique_ptr<LCompilers::MLIRModule> m;
+    Result<std::unique_ptr<MLIRModule>> res = asr_to_mlir(al, asr, diagnostics);
+    if (res.ok) {
+        m = std::move(res.result);
+    } else {
+        LCOMPILERS_ASSERT(diagnostics.has_error())
+        return res.error;
+    }
+
+    // MLIR -> LLVM
+    m->mlir_to_llvm();
+    return m;
+#else
+    throw LCompilersException("MLIR is not enabled");
+#endif
+}
+
 Result<std::string> FortranEvaluator::get_fortran(const std::string &code,
     LocationManager &lm, diag::Diagnostics &diagnostics)
 {
