@@ -166,16 +166,24 @@ bool Parser::parse(const std::string &input)
     if (!fixed_form) {
         m_tokenizer.set_string(inp);
         if (yyparse(*this) == 0) {
+            if (diag.has_error())
+                return false;
             return true;
         }
     } else {
         f_tokenizer.set_string(inp);
         if (!f_tokenizer.tokenize_input(diag, m_a)) return false;
         if (yyparse(*this) == 0) {
+            if (diag.has_error())
+                return false;
             return true;
         }
     }
-    throw parser_local::ParserError("Parsing unsuccessful (internal compiler error)");
+
+    if (!diag.has_error()) {
+        diag.add(parser_local::ParserError("Parsing unsuccessful (internal compiler error)").d);
+    }
+    return false;
 }
 
 Result<std::vector<int>> tokens(Allocator &al, const std::string &input,
@@ -378,8 +386,11 @@ void copy_rest_of_line(std::string &out, const std::string &s, size_t &pos,
             pos++;
         }
     }
-    out += s[pos]; // Copy the last `\n'
-    pos++;
+    // not always a program end's with '\n', but when it does, copy it
+    if (s[pos] == '\n') {
+        out += s[pos];
+        pos++;
+    }
 }
 
 // Checks that newlines are computed correctly
@@ -661,8 +672,11 @@ std::string token2text(const int token)
 
         T(TK_STRING, "string")
         T(TK_COMMENT, "comment")
+        T(TK_EOLCOMMENT, "end of line comment")
         T(TK_LABEL, "label")
-        T(TK_PRAGMA, "pragma")
+        T(TK_PRAGMA_DECL, "pragma declare")
+        T(TK_OMP, "pragma")
+        T(TK_OMP_END, "pragma end")
 
         T(TK_DBL_DOT, "..")
         T(TK_DBL_COLON, "::")
@@ -904,7 +918,7 @@ void Parser::handle_yyerror(const Location &loc, const std::string &msg)
             unsigned int invalid_token = this->f_tokenizer.token_pos;
             if (invalid_token == 0 || invalid_token > f_tokenizer.tokens.size()) {
                 message = "unknown error";
-                throw parser_local::ParserError(message, loc);
+                diag.add(parser_local::ParserError(message, loc).d);
             }
             invalid_token--;
             LCOMPILERS_ASSERT(invalid_token < f_tokenizer.tokens.size())
@@ -935,7 +949,7 @@ void Parser::handle_yyerror(const Location &loc, const std::string &msg)
     } else {
         message = "Internal Compiler Error: parser returned unknown error";
     }
-    throw parser_local::ParserError(message, loc);
+    diag.add(parser_local::ParserError(message, loc).d);
 }
 
 } // namespace LCompilers::LFortran
