@@ -9,6 +9,7 @@
 #include <libasr/modfile.h>
 #include <libasr/pass/pass_utils.h>
 #include <libasr/pass/intrinsic_function_registry.h>
+#include <libasr/pass/intrinsic_subroutine_registry.h>
 #include <libasr/pass/intrinsic_array_function_registry.h>
 
 namespace LCompilers {
@@ -414,8 +415,8 @@ void set_intrinsic(ASR::symbol_t* sym) {
             func_sym_type->m_abi = ASR::abiType::Intrinsic;
             break;
         }
-        case ASR::symbolType::StructType: {
-            ASR::StructType_t* derived_type_sym = ASR::down_cast<ASR::StructType_t>(sym);
+        case ASR::symbolType::Struct: {
+            ASR::Struct_t* derived_type_sym = ASR::down_cast<ASR::Struct_t>(sym);
             derived_type_sym->m_abi = ASR::abiType::Intrinsic;
             break;
         }
@@ -467,8 +468,8 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
                             ASR::asr_t* v_var, ASR::symbol_t *v,
                             ASR::symbol_t* member, SymbolTable* current_scope) {
     member = ASRUtils::symbol_get_past_external(member);
-    if (ASR::is_a<ASR::StructType_t>(*member)) {
-        ASR::StructType_t* member_variable = ASR::down_cast<ASR::StructType_t>(member);
+    if (ASR::is_a<ASR::Struct_t>(*member)) {
+        ASR::Struct_t* member_variable = ASR::down_cast<ASR::Struct_t>(member);
         ASR::symbol_t *mem_es = nullptr;
         std::string mem_name = "1_" + std::string(ASRUtils::symbol_name(member));
         if (current_scope->resolve_symbol(mem_name)) {
@@ -480,7 +481,7 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
                 nullptr, 0, member_variable->m_name, ASR::accessType::Public));
             current_scope->add_symbol(mem_name, mem_es);
         }
-        ASR::ttype_t* member_type = ASRUtils::TYPE(ASR::make_Struct_t(al,
+        ASR::ttype_t* member_type = ASRUtils::TYPE(ASR::make_StructType_t(al,
             member_variable->base.base.loc, mem_es));
         return ASR::make_StructInstanceMember_t(al, loc, ASRUtils::EXPR(v_var),
             mem_es, ASRUtils::fix_scoped_type(al, member_type, current_scope), nullptr);
@@ -493,8 +494,8 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
         ASR::dimension_t* m_dims = nullptr;
         size_t n_dims = ASRUtils::extract_dimensions_from_ttype(member_type, m_dims);
         switch( member_type_->type ) {
-            case ASR::ttypeType::Struct: {
-                ASR::Struct_t* der = ASR::down_cast<ASR::Struct_t>(member_type_);
+            case ASR::ttypeType::StructType: {
+                ASR::StructType_t* der = ASR::down_cast<ASR::StructType_t>(member_type_);
                 std::string der_type_name = ASRUtils::symbol_name(der->m_derived_type);
                 ASR::symbol_t* der_type_sym = current_scope->resolve_symbol(der_type_name);
                 if( der_type_sym == nullptr ) {
@@ -505,9 +506,9 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
                         ASR::ExternalSymbol_t* m_ext = ASR::down_cast<ASR::ExternalSymbol_t>(m_external);
                         m_external = m_ext->m_external;
                         module_name = m_ext->m_module_name;
-                    } else if( ASR::is_a<ASR::StructType_t>(*m_external) ) {
+                    } else if( ASR::is_a<ASR::Struct_t>(*m_external) ) {
                         ASR::symbol_t* asr_owner = ASRUtils::get_asr_owner(m_external);
-                        if( ASR::is_a<ASR::StructType_t>(*asr_owner) ||
+                        if( ASR::is_a<ASR::Struct_t>(*asr_owner) ||
                             ASR::is_a<ASR::Module_t>(*asr_owner) ) {
                             module_name = ASRUtils::symbol_name(asr_owner);
                         }
@@ -543,10 +544,10 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
                     } else {
                         der_ext = current_scope->get_symbol(mangled_name);
                     }
-                    ASR::asr_t* der_new = ASR::make_Struct_t(al, loc, der_ext);
+                    ASR::asr_t* der_new = ASR::make_StructType_t(al, loc, der_ext);
                     member_type_ = ASRUtils::TYPE(der_new);
                 } else if(ASR::is_a<ASR::ExternalSymbol_t>(*der_type_sym)) {
-                    member_type_ = ASRUtils::TYPE(ASR::make_Struct_t(al, loc, der_type_sym));
+                    member_type_ = ASRUtils::TYPE(ASR::make_StructType_t(al, loc, der_type_sym));
                 }
                 member_type = ASRUtils::make_Array_t_util(al, loc,
                     member_type_, m_dims, n_dims);
@@ -586,14 +587,14 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
     const std::function<void (const std::string &, const Location &)> err) {
     ASR::ttype_t *left_type = ASRUtils::expr_type(left);
     ASR::ttype_t *right_type = ASRUtils::expr_type(right);
-    ASR::StructType_t *left_struct = nullptr;
-    if ( ASR::is_a<ASR::Struct_t>(*left_type) ) {
-        left_struct = ASR::down_cast<ASR::StructType_t>(
-            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::Struct_t>(
+    ASR::Struct_t *left_struct = nullptr;
+    if ( ASR::is_a<ASR::StructType_t>(*left_type) ) {
+        left_struct = ASR::down_cast<ASR::Struct_t>(
+            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::StructType_t>(
             left_type)->m_derived_type));
-    } else if ( ASR::is_a<ASR::Class_t>(*left_type) ) {
-        left_struct = ASR::down_cast<ASR::StructType_t>(
-            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::Class_t>(
+    } else if ( ASR::is_a<ASR::ClassType_t>(*left_type) ) {
+        left_struct = ASR::down_cast<ASR::Struct_t>(
+            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::ClassType_t>(
             left_type)->m_class_type));
     }
     bool found = false;
@@ -650,17 +651,24 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                             }
                             ASR::ttype_t *return_type = nullptr;
                             if( ASRUtils::get_FunctionType(func)->m_elemental &&
-                                func->n_args == 1 &&
+                                func->n_args >= 1 &&
                                 ASRUtils::is_array(ASRUtils::expr_type(a_args[0].m_value)) ) {
-                                return_type = ASRUtils::duplicate_type(al, ASRUtils::expr_type(a_args[0].m_value));
+                                ASR::dimension_t* array_dims;
+                                size_t array_n_dims = ASRUtils::extract_dimensions_from_ttype(
+                                ASRUtils::expr_type(a_args[0].m_value), array_dims);
+                                Vec<ASR::dimension_t> new_dims;
+                                new_dims.from_pointer_n_copy(al, array_dims, array_n_dims);
+                                return_type = ASRUtils::duplicate_type(al,
+                                                ASRUtils::get_FunctionType(func)->m_return_var_type,
+                                                &new_dims);
                             } else {
                                 return_type = ASRUtils::expr_type(func->m_return_var);
                                 bool is_array = ASRUtils::is_array(return_type);
                                 ASR::dimension_t* m_dims = nullptr;
                                 size_t n_dims = ASRUtils::extract_dimensions_from_ttype(return_type, m_dims);
                                 return_type = ASRUtils::type_get_past_array(return_type);
-                                if( ASR::is_a<ASR::Struct_t>(*return_type) ) {
-                                    ASR::Struct_t* struct_t = ASR::down_cast<ASR::Struct_t>(return_type);
+                                if( ASR::is_a<ASR::StructType_t>(*return_type) ) {
+                                    ASR::StructType_t* struct_t = ASR::down_cast<ASR::StructType_t>(return_type);
                                     if( curr_scope->get_counter() !=
                                         ASRUtils::symbol_parent_symtab(struct_t->m_derived_type)->get_counter() &&
                                         !curr_scope->resolve_symbol(ASRUtils::symbol_name(struct_t->m_derived_type)) ) {
@@ -670,7 +678,7 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                                                 ASRUtils::symbol_name(ASRUtils::get_asr_owner(struct_t->m_derived_type)), nullptr, 0,
                                                 ASRUtils::symbol_name(struct_t->m_derived_type), ASR::accessType::Public)));
                                     }
-                                    return_type = ASRUtils::TYPE(ASR::make_Struct_t(al, loc,
+                                    return_type = ASRUtils::TYPE(ASR::make_StructType_t(al, loc,
                                         curr_scope->resolve_symbol(ASRUtils::symbol_name(struct_t->m_derived_type))));
                                     if( is_array ) {
                                         return_type = ASRUtils::make_Array_t_util(al, loc, return_type, m_dims, n_dims);
@@ -685,7 +693,8 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                             asr = ASRUtils::make_FunctionCall_t_util(al, loc, a_name, sym,
                                                             a_args.p, 2,
                                                             return_type,
-                                                            nullptr, nullptr);
+                                                            nullptr, nullptr,
+                                                            false);
                         }
                     }
                     break;
@@ -734,17 +743,24 @@ void process_overloaded_unary_minus_function(ASR::symbol_t* proc, ASR::expr_t* o
             }
             ASR::ttype_t *return_type = nullptr;
             if( ASRUtils::get_FunctionType(func)->m_elemental &&
-                func->n_args == 1 &&
+                func->n_args >= 1 &&
                 ASRUtils::is_array(ASRUtils::expr_type(a_args[0].m_value)) ) {
-                return_type = ASRUtils::duplicate_type(al, ASRUtils::expr_type(a_args[0].m_value));
+                ASR::dimension_t* array_dims;
+                size_t array_n_dims = ASRUtils::extract_dimensions_from_ttype(
+                ASRUtils::expr_type(a_args[0].m_value), array_dims);
+                Vec<ASR::dimension_t> new_dims;
+                new_dims.from_pointer_n_copy(al, array_dims, array_n_dims);
+                return_type = ASRUtils::duplicate_type(al,
+                                ASRUtils::get_FunctionType(func)->m_return_var_type,
+                                &new_dims);
             } else {
                 return_type = ASRUtils::expr_type(func->m_return_var);
                 bool is_array = ASRUtils::is_array(return_type);
                 ASR::dimension_t* m_dims = nullptr;
                 size_t n_dims = ASRUtils::extract_dimensions_from_ttype(return_type, m_dims);
                 return_type = ASRUtils::type_get_past_array(return_type);
-                if( ASR::is_a<ASR::Struct_t>(*return_type) ) {
-                    ASR::Struct_t* struct_t = ASR::down_cast<ASR::Struct_t>(return_type);
+                if( ASR::is_a<ASR::StructType_t>(*return_type) ) {
+                    ASR::StructType_t* struct_t = ASR::down_cast<ASR::StructType_t>(return_type);
                     if( curr_scope->get_counter() !=
                         ASRUtils::symbol_parent_symtab(struct_t->m_derived_type)->get_counter() &&
                         !curr_scope->resolve_symbol(ASRUtils::symbol_name(struct_t->m_derived_type)) ) {
@@ -754,7 +770,7 @@ void process_overloaded_unary_minus_function(ASR::symbol_t* proc, ASR::expr_t* o
                                 ASRUtils::symbol_name(ASRUtils::get_asr_owner(struct_t->m_derived_type)), nullptr, 0,
                                 ASRUtils::symbol_name(struct_t->m_derived_type), ASR::accessType::Public)));
                     }
-                    return_type = ASRUtils::TYPE(ASR::make_Struct_t(al, loc,
+                    return_type = ASRUtils::TYPE(ASR::make_StructType_t(al, loc,
                         curr_scope->resolve_symbol(ASRUtils::symbol_name(struct_t->m_derived_type))));
                     if( is_array ) {
                         return_type = ASRUtils::make_Array_t_util(
@@ -770,7 +786,8 @@ void process_overloaded_unary_minus_function(ASR::symbol_t* proc, ASR::expr_t* o
             asr = ASRUtils::make_FunctionCall_t_util(al, loc, a_name, proc,
                                             a_args.p, 1,
                                             return_type,
-                                            nullptr, nullptr);
+                                            nullptr, nullptr,
+                                            false);
         }
     }
 }
@@ -783,14 +800,14 @@ bool use_overloaded_unary_minus(ASR::expr_t* operand,
     ASR::ttype_t *operand_type = ASRUtils::expr_type(operand);
     ASR::symbol_t* sym = curr_scope->resolve_symbol("~sub");
     if( !sym ) {
-        if( ASR::is_a<ASR::Struct_t>(*operand_type) ) {
-            ASR::Struct_t* struct_t = ASR::down_cast<ASR::Struct_t>(operand_type);
+        if( ASR::is_a<ASR::StructType_t>(*operand_type) ) {
+            ASR::StructType_t* struct_t = ASR::down_cast<ASR::StructType_t>(operand_type);
             ASR::symbol_t* struct_t_sym = ASRUtils::symbol_get_past_external(struct_t->m_derived_type);
-            if( ASR::is_a<ASR::StructType_t>(*struct_t_sym) ) {
-                ASR::StructType_t* struct_type_t = ASR::down_cast<ASR::StructType_t>(struct_t_sym);
+            if( ASR::is_a<ASR::Struct_t>(*struct_t_sym) ) {
+                ASR::Struct_t* struct_type_t = ASR::down_cast<ASR::Struct_t>(struct_t_sym);
                 sym = struct_type_t->m_symtab->resolve_symbol("~sub");
                 while( sym == nullptr && struct_type_t->m_parent != nullptr ) {
-                    struct_type_t = ASR::down_cast<ASR::StructType_t>(
+                    struct_type_t = ASR::down_cast<ASR::Struct_t>(
                         ASRUtils::symbol_get_past_external(struct_type_t->m_parent));
                     sym = struct_type_t->m_symtab->resolve_symbol("~sub");
                 }
@@ -836,7 +853,7 @@ bool use_overloaded_unary_minus(ASR::expr_t* operand,
 }
 
 bool is_op_overloaded(ASR::binopType op, std::string& intrinsic_op_name,
-                      SymbolTable* curr_scope, ASR::StructType_t* left_struct) {
+                      SymbolTable* curr_scope, ASR::Struct_t* left_struct) {
     bool result = true;
     switch(op) {
         case ASR::binopType::Add: {
@@ -958,14 +975,14 @@ bool use_overloaded_assignment(ASR::expr_t* target, ASR::expr_t* value,
     ASR::symbol_t* sym = curr_scope->resolve_symbol("~assign");
     ASR::expr_t* expr_dt = nullptr;
     if( !sym ) {
-        if( ASR::is_a<ASR::Struct_t>(*target_type) ) {
-            ASR::StructType_t* target_struct = ASR::down_cast<ASR::StructType_t>(
-                ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::Struct_t>(target_type)->m_derived_type));
+        if( ASR::is_a<ASR::StructType_t>(*target_type) ) {
+            ASR::Struct_t* target_struct = ASR::down_cast<ASR::Struct_t>(
+                ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::StructType_t>(target_type)->m_derived_type));
             sym = target_struct->m_symtab->resolve_symbol("~assign");
             expr_dt = target;
-        } else if( ASR::is_a<ASR::Struct_t>(*value_type) ) {
-            ASR::StructType_t* value_struct = ASR::down_cast<ASR::StructType_t>(
-                ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::Struct_t>(value_type)->m_derived_type));
+        } else if( ASR::is_a<ASR::StructType_t>(*value_type) ) {
+            ASR::Struct_t* value_struct = ASR::down_cast<ASR::Struct_t>(
+                ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::StructType_t>(value_type)->m_derived_type));
             sym = value_struct->m_symtab->resolve_symbol("~assign");
             expr_dt = value;
         }
@@ -1001,6 +1018,102 @@ bool use_overloaded_assignment(ASR::expr_t* target, ASR::expr_t* value,
     return found;
 }
 
+void process_overloaded_read_write_function(std::string &read_write, ASR::symbol_t* proc, Vec<ASR::expr_t*> &args,
+    ASR::ttype_t* arg_type, bool& found, Allocator& al, const Location& arg_loc,
+    SymbolTable* curr_scope, SetChar& current_function_dependencies,
+    SetChar& current_module_dependencies, ASR::asr_t*& asr, ASR::symbol_t* sym, const Location& loc, ASR::expr_t* expr_dt,
+    const std::function<void (const std::string &, const Location &)> err, char* pass_arg=nullptr) {
+    ASR::Function_t* subrout = ASR::down_cast<ASR::Function_t>(proc);
+    std::string matched_subrout_name = "";
+    ASR::ttype_t* func_arg_type = ASRUtils::expr_type(subrout->m_args[0]);
+    if( ASRUtils::types_equal(func_arg_type, arg_type) ) {
+        std::string arg0_name = ASRUtils::symbol_name(ASR::down_cast<ASR::Var_t>(subrout->m_args[0])->m_v);
+        if( pass_arg != nullptr ) {
+            std::string pass_arg_str = std::string(pass_arg);
+            if( (arg0_name == pass_arg_str && args[0] != expr_dt) ) {
+                err(std::string(subrout->m_name) + " is not a procedure of " +
+                    ASRUtils::type_to_str(arg_type),
+                    loc);
+            }
+        }
+        found = true;
+        Vec<ASR::call_arg_t> a_args;
+        a_args.reserve(al, args.size());
+        for (size_t i = 0; i < args.size(); i++) {
+            ASR::call_arg_t arg;
+            arg.loc = arg_loc;
+            arg.m_value = args[i];
+            a_args.push_back(al, arg);
+        }
+        std::string subrout_name = to_lower(subrout->m_name);
+        if( curr_scope->resolve_symbol(subrout_name) ) {
+            matched_subrout_name = subrout_name;
+        } else {
+            std::string mangled_name = subrout_name + "@" + read_write;
+            matched_subrout_name = mangled_name;
+        }
+        ASR::symbol_t *a_name = curr_scope->resolve_symbol(matched_subrout_name);
+        if( a_name == nullptr ) {
+            err("Unable to resolve matched subroutine for read/write overloading, " + matched_subrout_name, loc);
+        }
+        if (ASRUtils::symbol_parent_symtab(a_name)->get_counter() != curr_scope->get_counter()) {
+            ADD_ASR_DEPENDENCIES_WITH_NAME(curr_scope, a_name, current_function_dependencies, s2c(al, matched_subrout_name));
+        }
+        ASRUtils::insert_module_dependency(a_name, al, current_module_dependencies);
+        ASRUtils::set_absent_optional_arguments_to_null(a_args, subrout, al);
+        asr = ASRUtils::make_SubroutineCall_t_util(al, loc, a_name, sym,
+                                        a_args.p, a_args.n, nullptr, nullptr, false, false);
+    }
+}
+
+bool use_overloaded_file_read_write(std::string &read_write, Vec<ASR::expr_t*> args,
+                               SymbolTable* curr_scope, ASR::asr_t*& asr,
+                               Allocator &al, const Location& loc,
+                               SetChar& current_function_dependencies,
+                               SetChar& current_module_dependencies,
+                               const std::function<void (const std::string &, const Location &)> err) {
+    ASR::ttype_t *arg_type = ASRUtils::type_get_past_allocatable(ASRUtils::expr_type(args[0]));
+    bool found = false;
+    ASR::symbol_t* sym = curr_scope->resolve_symbol(read_write);
+    ASR::expr_t* expr_dt = nullptr;
+    if( sym == nullptr ) {
+        if( ASR::is_a<ASR::StructType_t>(*arg_type) ) {
+            ASR::Struct_t* arg_struct = ASR::down_cast<ASR::Struct_t>(
+                ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::StructType_t>(arg_type)->m_derived_type));
+            sym = arg_struct->m_symtab->resolve_symbol(read_write);
+            expr_dt = args[0];
+        }
+    } else {
+        ASR::symbol_t* orig_sym = ASRUtils::symbol_get_past_external(sym);
+        ASR::CustomOperator_t* gen_proc = ASR::down_cast<ASR::CustomOperator_t>(orig_sym);
+        for( size_t i = 0; i < gen_proc->n_procs && !found; i++ ) {
+            ASR::symbol_t* proc = ASRUtils::symbol_get_past_external(gen_proc->m_procs[i]);
+            switch( proc->type ) {
+                case ASR::symbolType::Function: {
+                    process_overloaded_read_write_function(read_write, proc, args, arg_type,
+                        found, al, args[0]->base.loc, curr_scope,
+                        current_function_dependencies, current_module_dependencies, asr, sym,
+                        loc, expr_dt, err);
+                    break;
+                }
+                case ASR::symbolType::ClassProcedure: {
+                    ASR::ClassProcedure_t* class_proc = ASR::down_cast<ASR::ClassProcedure_t>(proc);
+                    ASR::symbol_t* proc_func = ASR::down_cast<ASR::ClassProcedure_t>(proc)->m_proc;
+                    process_overloaded_read_write_function(read_write, proc_func, args, arg_type,
+                        found, al, args[0]->base.loc, curr_scope,
+                        current_function_dependencies, current_module_dependencies, asr, proc_func, loc,
+                        expr_dt, err, class_proc->m_self_argument);
+                    break;
+                }
+                default: {
+                    err("Only functions and class procedures can be used for generic read/write statement, found " + std::to_string(proc->type), loc);
+                }
+            }
+        }
+    }
+    return found;
+}
+
 bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                     ASR::cmpopType op, std::string& intrinsic_op_name,
                     SymbolTable* curr_scope, ASR::asr_t*& asr,
@@ -1010,14 +1123,14 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                     const std::function<void (const std::string &, const Location &)> err) {
     ASR::ttype_t *left_type = ASRUtils::expr_type(left);
     ASR::ttype_t *right_type = ASRUtils::expr_type(right);
-    ASR::StructType_t *left_struct = nullptr;
-    if ( ASR::is_a<ASR::Struct_t>(*left_type) ) {
-        left_struct = ASR::down_cast<ASR::StructType_t>(
-            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::Struct_t>(
+    ASR::Struct_t *left_struct = nullptr;
+    if ( ASR::is_a<ASR::StructType_t>(*left_type) ) {
+        left_struct = ASR::down_cast<ASR::Struct_t>(
+            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::StructType_t>(
             left_type)->m_derived_type));
-    } else if ( ASR::is_a<ASR::Class_t>(*left_type) ) {
-        left_struct = ASR::down_cast<ASR::StructType_t>(
-            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::Class_t>(
+    } else if ( ASR::is_a<ASR::ClassType_t>(*left_type) ) {
+        left_struct = ASR::down_cast<ASR::Struct_t>(
+            ASRUtils::symbol_get_past_external(ASR::down_cast<ASR::ClassType_t>(
             left_type)->m_class_type));
     }
     bool found = false;
@@ -1046,10 +1159,10 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                         ASR::ttype_t* right_arg_type = ASRUtils::expr_type(func->m_args[1]);
                         if( (left_arg_type->type == left_type->type &&
                             right_arg_type->type == right_type->type)
-                        || (ASR::is_a<ASR::Class_t>(*left_arg_type) &&
-                            ASR::is_a<ASR::Struct_t>(*left_type))
-                        || (ASR::is_a<ASR::Class_t>(*right_arg_type) &&
-                            ASR::is_a<ASR::Struct_t>(*right_type))) {
+                        || (ASR::is_a<ASR::ClassType_t>(*left_arg_type) &&
+                            ASR::is_a<ASR::StructType_t>(*left_type))
+                        || (ASR::is_a<ASR::ClassType_t>(*right_arg_type) &&
+                            ASR::is_a<ASR::StructType_t>(*right_type))) {
                             found = true;
                             Vec<ASR::call_arg_t> a_args;
                             a_args.reserve(al, 2);
@@ -1071,9 +1184,16 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                             }
                             ASR::ttype_t *return_type = nullptr;
                             if( ASRUtils::get_FunctionType(func)->m_elemental &&
-                                func->n_args == 1 &&
+                                func->n_args >= 1 &&
                                 ASRUtils::is_array(ASRUtils::expr_type(a_args[0].m_value)) ) {
-                                return_type = ASRUtils::duplicate_type(al, ASRUtils::expr_type(a_args[0].m_value));
+                                ASR::dimension_t* array_dims;
+                                size_t array_n_dims = ASRUtils::extract_dimensions_from_ttype(
+                                ASRUtils::expr_type(a_args[0].m_value), array_dims);
+                                Vec<ASR::dimension_t> new_dims;
+                                new_dims.from_pointer_n_copy(al, array_dims, array_n_dims);
+                                return_type = ASRUtils::duplicate_type(al,
+                                                ASRUtils::get_FunctionType(func)->m_return_var_type,
+                                                &new_dims);
                             } else {
                                 return_type = ASRUtils::expr_type(func->m_return_var);
                             }
@@ -1085,7 +1205,8 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                             asr = ASRUtils::make_FunctionCall_t_util(al, loc, a_name, sym,
                                                             a_args.p, 2,
                                                             return_type,
-                                                            nullptr, nullptr);
+                                                            nullptr, nullptr,
+                                                            false);
                         }
                     }
                     break;
@@ -1101,7 +1222,7 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
 }
 
 bool is_op_overloaded(ASR::cmpopType op, std::string& intrinsic_op_name,
-                      SymbolTable* curr_scope, ASR::StructType_t *left_struct) {
+                      SymbolTable* curr_scope, ASR::Struct_t *left_struct) {
     bool result = true;
     switch(op) {
         case ASR::cmpopType::Eq: {
@@ -1169,7 +1290,7 @@ bool argument_types_match(const Vec<ASR::call_arg_t>& args,
             // Otherwise this should not be nullptr
             ASR::ttype_t *arg1 = ASRUtils::expr_type(args[i].m_value);
             ASR::ttype_t *arg2 = v->m_type;
-            if (!types_equal(arg1, arg2)) {
+            if (!types_equal(arg1, arg2, !ASRUtils::get_FunctionType(sub)->m_elemental)) {
                 return false;
             }
         }
@@ -1220,9 +1341,16 @@ ASR::asr_t* symbol_resolve_external_generic_procedure_without_eval(
         func = ASR::down_cast<ASR::Function_t>(final_sym);
         if (func->m_return_var) {
             if( ASRUtils::get_FunctionType(func)->m_elemental &&
-                func->n_args == 1 &&
+                func->n_args >= 1 &&
                 ASRUtils::is_array(ASRUtils::expr_type(args[0].m_value)) ) {
-                return_type = ASRUtils::duplicate_type(al, ASRUtils::expr_type(args[0].m_value));
+                ASR::dimension_t* array_dims;
+                size_t array_n_dims = ASRUtils::extract_dimensions_from_ttype(
+                ASRUtils::expr_type(args[0].m_value), array_dims);
+                Vec<ASR::dimension_t> new_dims;
+                new_dims.from_pointer_n_copy(al, array_dims, array_n_dims);
+                return_type = ASRUtils::duplicate_type(al,
+                                ASRUtils::get_FunctionType(func)->m_return_var_type,
+                                &new_dims);
             } else {
                 return_type = ASRUtils::EXPR2VAR(func->m_return_var)->m_type;
             }
@@ -1267,7 +1395,8 @@ ASR::asr_t* symbol_resolve_external_generic_procedure_without_eval(
         return ASRUtils::make_FunctionCall_t_util(al, loc, final_sym,
                                         v, args.p, args.size(),
                                         return_type,
-                                        nullptr, nullptr);
+                                        nullptr, nullptr,
+                                        false);
     }
 }
 
@@ -1319,7 +1448,9 @@ ASR::asr_t* make_Cast_t_value(Allocator &al, const Location &a_loc,
                                 ASRUtils::expr_value(a_arg))->m_n;
             value = ASR::down_cast<ASR::expr_t>(ASR::make_UnsignedIntegerConstant_t(al, a_loc, int_value, a_type));
         } else if (a_kind == ASR::cast_kindType::IntegerToLogical) {
-            // TODO: implement
+            int64_t int_value = ASR::down_cast<ASR::IntegerConstant_t>(
+                                ASRUtils::expr_value(a_arg))->m_n;
+            value = ASR::down_cast<ASR::expr_t>(ASR::make_LogicalConstant_t(al, a_loc, int_value, a_type));
         } else if (a_kind == ASR::cast_kindType::ComplexToComplex) {
             ASR::ComplexConstant_t* value_complex = ASR::down_cast<ASR::ComplexConstant_t>(
                         ASRUtils::expr_value(a_arg));
@@ -1338,10 +1469,9 @@ ASR::asr_t* make_Cast_t_value(Allocator &al, const Location &a_loc,
             args.reserve(al, 1);
             args.push_back(al, a_arg);
             LCompilers::ASRUtils::create_intrinsic_function create_function =
-                LCompilers::ASRUtils::IntrinsicScalarFunctionRegistry::get_create_function("SymbolicInteger");
-            value = ASR::down_cast<ASR::expr_t>(create_function(al, a_loc, args,
-                [](const std::string&, const Location&) {
-            }));
+                LCompilers::ASRUtils::IntrinsicElementalFunctionRegistry::get_create_function("SymbolicInteger");
+            diag::Diagnostics diag;
+            value = ASR::down_cast<ASR::expr_t>(create_function(al, a_loc, args, diag));
         }
     }
 
@@ -1409,7 +1539,7 @@ ASR::asr_t* make_Binop_util(Allocator &al, const Location& loc, ASR::binopType b
                 ASRUtils::duplicate_type(al, ttype), nullptr);
         }
         default:
-            throw LCompilersException("Not implemented " + std::to_string(ttype->type));
+            throw LCompilersException("Not implemented " + ASRUtils::type_to_str_python(ttype));
     }
 }
 
@@ -1430,7 +1560,7 @@ ASR::asr_t* make_Cmpop_util(Allocator &al, const Location& loc, ASR::cmpopType c
             return ASR::make_StringCompare_t(al, loc, lexpr, cmpop, rexpr, expr_type, nullptr);
         }
         default:
-            throw LCompilersException("Not implemented " + std::to_string(ttype->type));
+            throw LCompilersException("Not implemented " + ASRUtils::type_to_str_python(ttype));
     }
 }
 
@@ -1454,7 +1584,7 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
     dims.push_back(al, dim);
     ASR::ttype_t* dest_shape_type = ASRUtils::TYPE(ASR::make_Array_t(al, loc,
         ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)), dims.p, dims.size(),
-        is_value_character_array ? ASR::array_physical_typeType::CharacterArraySinglePointer: ASR::array_physical_typeType::FixedSizeArray));
+        is_value_character_array && !is_value_constant(expr2) ? ASR::array_physical_typeType::CharacterArraySinglePointer: ASR::array_physical_typeType::FixedSizeArray));
 
     ASR::expr_t* dest_shape = nullptr;
     ASR::expr_t* value = nullptr;
@@ -1464,8 +1594,8 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
         for( size_t i = 0; i < expr1_ndims; i++ ) {
             lengths.push_back(al, ASRUtils::expr_value(expr1_mdims[i].m_length));
         }
-        dest_shape = ASRUtils::EXPR(ASR::make_ArrayConstant_t(al, loc,
-            lengths.p, lengths.size(), dest_shape_type, ASR::arraystorageType::ColMajor));
+        dest_shape = EXPR(ASRUtils::make_ArrayConstructor_t_util(al, loc, lengths.p,
+            lengths.size(), dest_shape_type, ASR::arraystorageType::ColMajor));
         Vec<ASR::dimension_t> dims;
         dims.reserve(al, 1);
         ASR::dimension_t dim;
@@ -1481,14 +1611,17 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
             ASRUtils::get_fixed_size_of_array(expr1_mdims, expr1_ndims) <= 256 ) {
             ASR::ttype_t* value_type = ASRUtils::TYPE(ASR::make_Array_t(al, loc,
                 ASRUtils::type_get_past_array(ASRUtils::expr_type(expr2)), dims.p, dims.size(),
-                is_value_character_array ? ASR::array_physical_typeType::CharacterArraySinglePointer: ASR::array_physical_typeType::FixedSizeArray));
+                is_value_character_array && !ASRUtils::is_value_constant(expr2) ? ASR::array_physical_typeType::CharacterArraySinglePointer: ASR::array_physical_typeType::FixedSizeArray));
             Vec<ASR::expr_t*> values;
             values.reserve(al, ASRUtils::get_fixed_size_of_array(expr1_mdims, expr1_ndims));
             for( int64_t i = 0; i < ASRUtils::get_fixed_size_of_array(expr1_mdims, expr1_ndims); i++ ) {
                 values.push_back(al, expr2);
             }
-            value = ASRUtils::EXPR(ASR::make_ArrayConstant_t(al, loc,
-                values.p, values.size(), value_type, ASR::arraystorageType::ColMajor));
+            value = EXPR(ASRUtils::make_ArrayConstructor_t_util(al, loc, values.p,
+                values.size(), value_type, ASR::arraystorageType::ColMajor));
+            if (ASR::is_a<ASR::ArrayConstructor_t>(*value) && ASRUtils::expr_value(value)) {
+                value = ASRUtils::expr_value(value);
+            }
             ret_type = value_type;
         }
     } else {
@@ -1572,6 +1705,12 @@ int64_t compute_leading_zeros(int64_t number, int64_t kind) {
     return leading_zeros;
 }
 
+void append_error(diag::Diagnostics& diag, const std::string& msg,
+                const Location& loc) {
+    diag.add(diag::Diagnostic(msg, diag::Level::Error,
+        diag::Stage::Semantic, {diag::Label("", { loc })}));
+}
+
 
 //Initialize pointer to zero so that it can be initialized in first call to get_instance
 ASRUtils::LabelGenerator* ASRUtils::LabelGenerator::label_generator = nullptr;
