@@ -4,6 +4,7 @@
 #include <memory>
 
 #include <llvm/IR/Value.h>
+#include <llvm/IR/Module.h>
 #include <llvm/IR/IRBuilder.h>
 #include <libasr/asr.h>
 
@@ -19,10 +20,6 @@
 
 namespace LCompilers {
 
-    #define get_builder0() llvm::BasicBlock &entry_block = builder->GetInsertBlock()->getParent()->getEntryBlock(); \
-        llvm::IRBuilder<> builder0(context); \
-        builder0.SetInsertPoint(&entry_block, entry_block.getFirstInsertionPt()); \
-
     // Platform dependent fast unique hash:
     static inline uint64_t get_hash(ASR::asr_t *node)
     {
@@ -74,7 +71,7 @@ namespace LCompilers {
         llvm::Function *fn_printf = module.getFunction("_lfortran_printf");
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
-                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8PtrTy(context)}, true);
+                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lfortran_printf", &module);
         }
@@ -87,22 +84,41 @@ namespace LCompilers {
         llvm::Function *fn_printf = module.getFunction("_lcompilers_string_format_fortran");
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
-                    llvm::Type::getInt8PtrTy(context),
+                    llvm::Type::getInt8Ty(context)->getPointerTo(),
                     {llvm::Type::getInt32Ty(context),
-                    llvm::Type::getInt8PtrTy(context)}, true);
+                    llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lcompilers_string_format_fortran", &module);
         }
         return builder.CreateCall(fn_printf, args);
     }
 
+    static inline llvm::Value* lfortran_str_copy(llvm::Value* dest, llvm::Value *src, bool is_allocatable,
+        llvm::Module &module, llvm::IRBuilder<> &builder, llvm::LLVMContext &context) {
+        std::string runtime_func_name = "_lfortran_strcpy";
+        llvm::Function *fn = module.getFunction(runtime_func_name);
+        if (!fn) {
+            llvm::FunctionType *function_type = llvm::FunctionType::get(
+                     llvm::Type::getVoidTy(context), {
+                        llvm::Type::getInt8Ty(context)->getPointerTo()->getPointerTo(),
+                        llvm::Type::getInt8Ty(context)->getPointerTo(),
+                        llvm::Type::getInt8Ty(context)
+                    }, false);
+            fn = llvm::Function::Create(function_type,
+                    llvm::Function::ExternalLinkage, runtime_func_name, module);
+        }
+        llvm::Value* free_string = llvm::ConstantInt::get(
+            llvm::Type::getInt8Ty(context), llvm::APInt(8, is_allocatable));
+        return builder.CreateCall(fn, {dest, src, free_string});
+    }
+
     static inline void print_error(llvm::LLVMContext &context, llvm::Module &module,
         llvm::IRBuilder<> &builder, const std::vector<llvm::Value*> &args)
     {
         llvm::Function *fn_printf = module.getFunction("_lcompilers_print_error");
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
-                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8PtrTy(context)}, true);
+                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lcompilers_print_error", &module);
         }
@@ -134,7 +150,7 @@ namespace LCompilers {
         if (!fn) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
                 llvm::Type::getVoidTy(context), {
-                    llvm::Type::getInt8PtrTy(context),
+                    llvm::Type::getInt8Ty(context)->getPointerTo(),
                     llvm::Type::getInt1Ty(context)
                 }, true);
             fn = llvm::Function::Create(function_type,
@@ -146,10 +162,7 @@ namespace LCompilers {
 
     namespace LLVM {
 
-        llvm::Value* CreateLoad(llvm::IRBuilder<> &builder, llvm::Value *x);
         llvm::Value* CreateStore(llvm::IRBuilder<> &builder, llvm::Value *x, llvm::Value *y);
-        llvm::Value* CreateGEP(llvm::IRBuilder<> &builder, llvm::Value *x, std::vector<llvm::Value *> &idx);
-        llvm::Value* CreateInBoundsGEP(llvm::IRBuilder<> &builder, llvm::Value *x, std::vector<llvm::Value *> &idx);
         llvm::Value* lfortran_malloc(llvm::LLVMContext &context, llvm::Module &module,
                 llvm::IRBuilder<> &builder, llvm::Value* arg_size);
         llvm::Value* lfortran_realloc(llvm::LLVMContext &context, llvm::Module &module,
@@ -161,8 +174,8 @@ namespace LCompilers {
         static inline bool is_llvm_struct(ASR::ttype_t* asr_type) {
             return ASR::is_a<ASR::Tuple_t>(*asr_type) ||
                    ASR::is_a<ASR::List_t>(*asr_type) ||
-                   ASR::is_a<ASR::Struct_t>(*asr_type) ||
-                   ASR::is_a<ASR::Class_t>(*asr_type)||
+                   ASR::is_a<ASR::StructType_t>(*asr_type) ||
+                   ASR::is_a<ASR::ClassType_t>(*asr_type)||
                    ASR::is_a<ASR::Dict_t>(*asr_type);
         }
         static inline bool is_llvm_pointer(const ASR::ttype_t& asr_type) {
@@ -200,6 +213,7 @@ namespace LCompilers {
             std::map<std::string, std::map<std::string, int>>& name2memidx;
             std::unordered_map<std::uint32_t, std::unordered_map<std::string, llvm::Type*>>& arr_arg_type_cache;
             std::map<std::string, std::pair<llvm::Type*, llvm::Type*>>& fname2arg_type;
+            std::map<llvm::Value *, llvm::Type *> &ptr_type;
 
             LLVMDictInterface* dict_api_lp;
             LLVMDictInterface* dict_api_sc;
@@ -221,16 +235,43 @@ namespace LCompilers {
                 std::map<std::string, std::map<std::string, int>>& name2memidx_,
                 CompilerOptions &compiler_options_,
                 std::unordered_map<std::uint32_t, std::unordered_map<std::string, llvm::Type*>>& arr_arg_type_cache_,
-                std::map<std::string, std::pair<llvm::Type*, llvm::Type*>>& fname2arg_type_);
+                std::map<std::string, std::pair<llvm::Type*, llvm::Type*>>& fname2arg_type_,
+                std::map<llvm::Value *, llvm::Type *> &ptr_type_);
 
             llvm::Value* create_gep(llvm::Value* ds, int idx);
 
+            llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, int idx);
+
             llvm::Value* create_gep(llvm::Value* ds, llvm::Value* idx);
 
+            llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, llvm::Value* idx);
+
             llvm::Value* create_ptr_gep(llvm::Value* ptr, int idx);
 
+            llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, int idx);
+
             llvm::Value* create_ptr_gep(llvm::Value* ptr, llvm::Value* idx);
 
+            llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, llvm::Value* idx);
+
+            llvm::Value* CreateLoad(llvm::Value *x);
+            llvm::Value* CreateLoad2(llvm::Type *t, llvm::Value *x);
+            llvm::Value* CreateLoad2(ASR::ttype_t *type, llvm::Value *x);
+            llvm::Value* CreateGEP(llvm::Value *x, std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateGEP2(llvm::Type *t, llvm::Value *x,
+                std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateGEP2(ASR::ttype_t *type, llvm::Value *x, int idx);
+            llvm::Value* CreateInBoundsGEP(llvm::Value *x, std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateInBoundsGEP2(llvm::Type *t, llvm::Value *x,
+                std::vector<llvm::Value *> &idx);
+
+            llvm::AllocaInst* CreateAlloca(llvm::Type* type,
+                llvm::Value* size=nullptr, std::string Name="",
+                bool is_llvm_ptr=false);
+            llvm::AllocaInst* CreateAlloca(llvm::IRBuilder<> &builder,
+                llvm::Type* type, llvm::Value* size=nullptr, std::string Name="",
+                bool is_llvm_ptr=false);
+
             llvm::Type* getIntType(int a_kind, bool get_pointer=false);
 
             void start_new_block(llvm::BasicBlock *bb);
@@ -250,9 +291,9 @@ namespace LCompilers {
             llvm::Type* getMemberType(ASR::ttype_t* mem_type,
                 ASR::Variable_t* member, llvm::Module* module);
 
-            void createStructContext(ASR::StructType_t* der_type);
+            void createStructTypeContext(ASR::Struct_t* der_type);
 
-            llvm::Type* getStructType(ASR::StructType_t* der_type, llvm::Module* module, bool is_pointer=false);
+            llvm::Type* getStructType(ASR::Struct_t* der_type, llvm::Module* module, bool is_pointer=false);
 
             llvm::Type* getStructType(ASR::ttype_t* _type, llvm::Module* module, bool is_pointer=false);
 
@@ -262,9 +303,9 @@ namespace LCompilers {
             llvm::Type* getUnionType(ASR::ttype_t* _type,
                 llvm::Module* module, bool is_pointer=false);
 
-            llvm::Type* getClassType(ASR::ClassType_t* der_type, bool is_pointer=false);
+            llvm::Type* getClassType(ASR::Class_t* der_type, bool is_pointer=false);
 
-            llvm::Type* getClassType(ASR::StructType_t* der_type, bool is_pointer=false);
+            llvm::Type* getClassType(ASR::Struct_t* der_type, bool is_pointer=false);
 
             llvm::Type* getClassType(ASR::ttype_t* _type, bool is_pointer=false);
 
@@ -460,7 +501,7 @@ namespace LCompilers {
 
             llvm::LLVMContext& context;
             LLVMUtils* llvm_utils;
-            llvm::IRBuilder<>* builder;
+            // llvm::IRBuilder<>* builder;
 
             std::map<std::string, std::pair<llvm::Type*, size_t>> typecode2tupletype;
 
