@@ -243,7 +243,9 @@ public:
         if( ASRUtils::is_intrinsic_function2(func) ||
                 std::string(func->m_name) == current_routine ||
                 // Never Inline BindC Function
-                ASRUtils::get_FunctionType(func)->m_abi == ASR::abiType::BindC) {
+                ASRUtils::get_FunctionType(func)->m_abi == ASR::abiType::BindC || 
+                // Never Inline Interface Function
+                ASRUtils::get_FunctionType(func)->m_deftype == ASR::deftypeType::Interface) {
             return ;
         }
 
@@ -313,8 +315,8 @@ public:
             if( !ASR::is_a<ASR::Variable_t>(*itr.second) ||
                  ASRUtils::is_character(*ASRUtils::symbol_type(itr.second)) ||
                  ASRUtils::is_array(ASRUtils::symbol_type(itr.second)) ||
-                 ASR::is_a<ASR::Struct_t>(*ASRUtils::symbol_type(itr.second)) ||
-                 ASR::is_a<ASR::Class_t>(*ASRUtils::symbol_type(itr.second)) ) {
+                 ASR::is_a<ASR::StructType_t>(*ASRUtils::symbol_type(itr.second)) ||
+                 ASR::is_a<ASR::ClassType_t>(*ASRUtils::symbol_type(itr.second)) ) {
                 arg2value.clear();
                 return ;
             }
